## Updated Filter Junction code to normalize based on gene expression from RSEM
## This version from Oct 2, 2014 uses RSEM data to normalize the junction expression
## It uses the ENTREZID rather than gene symbols to align junction and RSEM data. 
## This version updated Nov 11,2014 removes log transformation until graphing.  junc.RPM rather than junc.log.RPM
## December 12, 2014, cleaned up code, added a plotting function
## Mar 23, 2015 -- apply this algorithm to TCGA data

## Load needed librarys from Bioclite
library('Homo.sapiens')
library('org.Hs.eg.db')
library('GenomicRanges')

## this is non-log transformed data and includes pheno (RAW, in RPM)
TCGA.HNSC<-read.table(file='C:/Users/tguo5/Documents/Firehose/HNSCextracted/HNSC.rnaseqv2__illuminahiseq_rnaseqv2__unc_edu__Level_3__junction_quantification__data.data.txt', sep='\t', header=T, stringsAsFactors = F)
load('C:/Users/tguo5/Dropbox/SpliceFusions/JuncRPM_11Nov2014.RDa')
## includes phenotype information
# pheno<-read.table(file='~/pheno92414.txt', sep='\t', header = T, stringsAsFactors = F)

## remove unwanted samples (only tumors and normals, exclude cell lines, xenografts etc.)
pheno<-pheno[pheno$classes == "Tumor" | pheno$classes == "Normal",]
row.names(pheno)<-pheno$junctionSample

# align phenotype names and expression data - removes excluded samples
junc.RPM<-junc.RPM[,pheno$junctionSample]

## now we will filter the putative junctions
### filter genes by cut off of 
fcCutoff=10
Cutoff.ratio<-1-(1/fcCutoff)
# enforce overall Fold change
logFC <- apply(junc.RPM,1,function(x){(max(x)-min(x))/max(x)}) > Cutoff.ratio 
junc.RPM <- junc.RPM[which(logFC),]

rm(logFC)
gc()

###### NEW FILTER: See if tumors have any outliers ###########################
##PHENO should have 'Normal' or 'Tumor' calls where Tumor ==1, Normal ==0, and names of each sample associated
PHENO<-pheno$classes=='Tumor'
PHENO<-as.numeric(PHENO)
names(PHENO)<-row.names(pheno)

## run the ogsa function
## get function
source(file='C:/Users/tguo5/Dropbox/SpliceVariantCalifano/Fusions/Fusions_OGSA_17Nov2014.R')

FirstFilter<-dotheogsa(Sample.data=junc.RPM, PHENO=PHENO, offsets=0.1)
has.outliers<-FirstFilter[,1]>1|FirstFilter[,3]>1
junc.RPM<-junc.RPM[has.outliers,]

### Filter all genes on the X and Y chromosomes
filterSex=T
if (filterSex) {
  junc.RPM <- junc.RPM[grep('chr[XY]',row.names(junc.RPM),value=T,invert=T),]
}

### Get the genomic information for all the junctions.  
### This is based on hg19 build

# Create GenomicRanges object for junctions
chr <- sapply(strsplit(row.names(junc.RPM),split=":"), function(x){x[[1]]})
start <- as.numeric(sapply(strsplit(row.names(junc.RPM),split="[:-]"),
                           function(x){x[[2]]}))
end <- as.numeric(sapply(strsplit(row.names(junc.RPM),split="[:-]"),
                         function(x){x[[3]]}))


geneAnnot <- GRanges(seqnames=Rle(chr),
                     IRanges(start=start, end=end))
names(geneAnnot) <- row.names(junc.RPM)

geneAnnotAll <- GRanges(seqnames=Rle(chr),
                        IRanges(start=start, end=end))
names(geneAnnotAll) <- row.names(junc.RPM)

# get GenomicRanges object with genes for whole human genome
gn <- genes(TxDb.Hsapiens.UCSC.hg19.knownGene)
gSymbol <- select(org.Hs.eg.db,keys=as.character(gn$gene_id),
                  columns=c('SYMBOL'),keytype='ENTREZID')
gn$SYMBOL <- gSymbol$SYMBOL
gn$ENTREZID <- gSymbol$ENTREZID
rm(gSymbol, chr, end, start)
gc()

# find symbols and ENTREZID for our junctions
overlap <- findOverlaps(geneAnnot,gn)

geneSYMBOLS <- tapply(gn$SYMBOL[subjectHits(overlap)],
                      queryHits(overlap),paste,collapse=';')

# add to genome ranges object
geneAnnot$SYMBOL <- NA
geneAnnot$SYMBOL[as.numeric(names(geneSYMBOLS))] <- geneSYMBOLS

geneENTREZID <- tapply(gn$ENTREZID[subjectHits(overlap)],
                       queryHits(overlap),paste,collapse=';')

geneAnnot$ENTREZID <- NA
geneAnnot$ENTREZID[as.numeric(names(geneENTREZID))] <- geneENTREZID

# get known exons
en <- exons(TxDb.Hsapiens.UCSC.hg19.knownGene)

## filtering for putative jucntions, skipping, insertion and deletion events based on known exons.
# annotate skipping events as those which overlap with more than two exons
exo <- findOverlaps(GRanges(seqnames(geneAnnot),
                            ranges=IRanges(start=start(geneAnnot)+1,
                                           end=end(geneAnnot)-1),
                            strand=strand(geneAnnot)),en,
                    type='within')
exoCount <- table(factor(queryHits(exo))) > 0
geneAnnot$skipping <- F
geneAnnot$skipping[as.numeric(names(exoCount))] <- exoCount

# insertion events start or end outside of known exons
geneAnnot.start <- GRanges(seqnames(geneAnnot),
                           ranges=IRanges(start=start(geneAnnot),
                                          end=start(geneAnnot)),
                           strand=strand(geneAnnot))
geneAnnot.end <- GRanges(seqnames(geneAnnot),
                         ranges=IRanges(start=end(geneAnnot),
                                        end=end(geneAnnot)),
                         strand=strand(geneAnnot))

geneAnnot$insertions <- !overlapsAny(geneAnnot.start, en) |
  !overlapsAny(geneAnnot.end, en)

# deletion events occur w/in an exon but not at its start or end
en.start <- GRanges(seqnames(en),
                    ranges=IRanges(start=start(en),
                                   end=start(en)),
                    strand=strand(en))
en.end <- GRanges(seqnames(en),
                  ranges=IRanges(start=end(en),
                                 end=end(en)),
                  strand=strand(en))
geneAnnot$deletions <- 
  (!overlapsAny(geneAnnot.end,en.start) & overlapsAny(geneAnnot.end,en)) | 
  (!overlapsAny(geneAnnot.start,en.end) & overlapsAny(geneAnnot.start,en))

filterToEvents<-T

if (filterToEvents) {
  geneAnnot <- geneAnnot[apply(cbind(geneAnnot$deletions,
                                     geneAnnot$insertions,
                                     geneAnnot$skipping),1,any),]
  junc.RPM <- junc.RPM[names(geneAnnot),]
}

## remove all that map to 'NA' no gene name
junc.RPM<-junc.RPM[!is.na(geneAnnot$SYMBOL),]

geneAnnot<-geneAnnot[row.names(junc.RPM),]

#Get RSEM Data for gene values for normalization.  Non-log transformed raw data
# HPVOPRSEMData <- read.table ('~/CalifanoHPVOP_RSEM_Genes_28Jul2014.txt', header=T, sep='\t', stringsAsFactors = F, row.names=1)

load('~/CalifanoHPVOP_RSEM_28Jul2014.RDa')

## remove unneeded samples from RSEM data, cell lines, xenografts and align samples with phenotypes
HPVOPRSEMData<-HPVOPRSEMData[ ,as.character(pheno$RSEMname)]

## remove row names for unknown genes containing "?" unknown genes
HPVOPRSEMData <- HPVOPRSEMData[-grep("\\?",row.names(HPVOPRSEMData)),]

## get gene names
HPVOPgenenames<-sapply(strsplit(row.names(HPVOPRSEMData), split ='\\|'), function(x){x[1]})
HPVOPEntrezID<-sapply(strsplit(row.names(HPVOPRSEMData), split ='\\|'), function(x){x[2]})
HPVOPgenenames[16272]<-'SLC35E2B'

###########################################Elana's code from 9/17/14
## try to use EntrezID to align genes
# line up RSEM gene names to reformatted gene names
names(HPVOPEntrezID) <- row.names(HPVOPRSEMData)

# create a list for each gene of all the junctions corresponding to that gene
tapply(names(geneAnnot),geneAnnot$ENTREZID,function(x){x}) -> genes2Junc
# tapply(names(geneAnnot),geneAnnot$SYMBOL,function(x){x}) -> genes2Junc_SYMB

# initialize matrix of values for each junction
junctionGeneRSEM <- matrix(0, nrow=length(geneAnnot), ncol=ncol(HPVOPRSEMData), dimnames=list(names(geneAnnot), colnames(HPVOPRSEMData)))

# get junctions corresponding to only one gene
singleGeneJunc <-  grep(names(genes2Junc),pattern=';',invert=T,value=T)

# if the gene is not in RSEM, set the value to NA
RSEMMissing <-  setdiff(singleGeneJunc, HPVOPEntrezID)
# junctionGeneRSEM[unlist(genes2Junc[RSEMMissing]),] <- NA
# 
# # set the actual RSEM values for those junctions
singleGeneJunc <- setdiff(singleGeneJunc, RSEMMissing) # delete all genes missing RSEM values
singleGeneJuncRSEMID <- HPVOPEntrezID[HPVOPEntrezID %in% singleGeneJunc]

for (g in names(singleGeneJuncRSEMID)) {
  junctionGeneRSEM[genes2Junc[[singleGeneJuncRSEMID[g]]], ] <- apply(HPVOPRSEMData[g,],2,rep,length(genes2Junc[[singleGeneJuncRSEMID[g]]]))
}

## clean up 
rm(HPVOPRSEMData, en, en.end, en.start, geneAnnot.end, geneAnnot.start)
gc()

################################################################
## Remove genes without RSEM data, not in true genes
## Assign gene names based on my reassignments, for junctions that align to multiple genes

junctionGeneRSEM_test<-junctionGeneRSEM

geneAssignments<-read.table(file='C:/Users/tguo5/Dropbox/SpliceVariantCalifano/Junctions/MultiGeneJunctionNames_5Dec14.txt', as.is=T, header=T, sep='\t', row.names=1)
## rename these junctions where i assigned the gene. 
for (i in row.names(geneAssignments))
{
  geneAnnot[i]$SYMBOL<-geneAssignments[i,'GeneAssignment']
}

for (g in row.names(geneAssignments)){
  if (is.na(geneAssignments[g,'GeneAssignment'])) next
  if (sum(HPVOPgenenames==geneAnnot[g]$SYMBOL)==0) next
index<-which(HPVOPgenenames==geneAnnot[g]$SYMBOL)
junctionGeneRSEM_test[g,]<-as.matrix(HPVOPRSEMData[index,])
}

junctionGeneRSEM<-junctionGeneRSEM_test
rm(junctionGeneRSEM_test)

## subset removing any genes without normalization
junc.RPM<-junc.RPM[!(apply(junctionGeneRSEM, 1,sum))==0,]

## Perform normalization using RSEM values

## Turn all the zeros in junctionGeneRSEM into 1.
## replace ALL zeros with 1.  Then when you divide it does not do anything.  
junctionGeneRSEM<-junctionGeneRSEM[row.names(junc.RPM),]
junctionGeneRSEM2<-junctionGeneRSEM
junctionGeneRSEM2[junctionGeneRSEM==0]<-1
junc.RPM.norm<-junc.RPM
junc.RPM.norm<-junc.RPM/junctionGeneRSEM2

############### Use OGSA for outlier ranking #################

##PHENO should have 'Normal' or 'Tumor' calls where Tumor ==1, Normal ==0, and names of each sample associated
PHENO<-pheno$classes=='Tumor'
PHENO<-as.numeric(PHENO)
names(PHENO)<-row.names(pheno)

## run the ogsa function
## get function
source(file='C:/Users/tguo5/Dropbox/SpliceFusions/OGSAfunctionwFisher.R')

allsamps=dotheogsa(Sample.data=junc.RPM.norm, PHENO=PHENO, offsets=0.00001, Fisher=T, correction='fdr')
## use default offset=0.001 for normalized data
# create lists of top genes
topgenelist10.ogsa=allsamps[,'var1']
topgenelist90.ogsa=allsamps[,'var2']

# Perform FDR adjustment on pvalues


##Use Fisher to subset candidates that have Fisher test p value <0.05, or cutoff can be adjusted
## Underexpression in tumors
toplist10<-allsamps[topgenelist10.ogsa,]
FisherP1.FDR<-p.adjust(toplist10[,'FisherP1'], method='fdr', n=nrow(toplist10))
toplist10<-toplist10[FisherP1.FDR<0.05,]
## Overexpression in tumors
toplist90<-allsamps[topgenelist90.ogsa,]
FisherP2.FDR<-p.adjust(toplist90[,'FisherP2'], method='fdr', n=nrow(toplist90))
toplist90<-toplist90[FisherP2.FDR<0.05,]

## Calculate median normalized value (of normals) to make a waterfall plot
## for RSEM
NORM.RSEM<-junctionGeneRSEM[,pheno$RSEMname[pheno$classes == 'Normal']]
## get median of normal expression
NORM.RSEM<-apply(NORM.RSEM, 1, median)

## get junction expression in normal samples only
NORM.RPM<-junc.RPM[,row.names(pheno)[pheno$classes == 'Normal']]
## get median of normal expression
NORM.RPM<-apply(NORM.RPM, 1, median)

## median of normal expression within the normalized data
NORM.RSEM.norm<-apply(junc.RPM.norm[,row.names(pheno)[pheno$classes == 'Normal']],1,median)

############# Make your plots
#### you can also try function created by PlotJunctionFunction.R
makepdf=T
pdffile='C:/Users/tguo5/Dropbox/SpliceVariantCalifano/Junctions/Junctions_ogsa10annot_15Dec.pdf'
tumcol='red'
normcol='blue'

## libraries needed for plot
library('NCBI2R')
library(gplots)

## putting topgenelist90 will get the ranked over-expression in tumors
## putting topgenelist90 will get the ranked under-expression in tumors
## [1:10] specify which/or how many to graph
junctions<-row.names(junc.RPM)[topgenelist10.copa[1:10]]

if (makepdf==T) {
  if (is.null(pdffile)) stop('Need to specify PDF file path name. pdffile=...')
  pdf(pdffile)
}

  count=0
  par(mfrow=c(2,2), oma=c(0,0,2,0))
  for (j in junctions)
  {
    ## first barplot is for raw junction data
    samples <- order(pheno$classes,junc.RPM.norm[j,],junc.RPM.norm[j,])
    barplot(log2(junc.RPM[j,samples]+1), cex.names=0.3, las=2, col = ifelse(pheno[samples,'classes']=='Tumor',tumcol,normcol))
    title(sprintf('Junction Expression (log)\nskip? %s del? %s? ins? %s',
                  geneAnnot[j,]$skipping, geneAnnot[j,]$deletions, 
                  geneAnnot[j,]$insertions),cex.main=0.7)
    
    ## this plot is for normalized junction expression - with RSEM.  
    barplot(log2(junc.RPM.norm[j,samples]+1)-log2(NORM.RSEM.norm[j]+1),
            las=2, cex.names=0.3,
            col=ifelse(pheno[samples,'classes']=='Tumor',tumcol,normcol))
    title('Junction expression \nNormalized by RSEM (log)', cex.main=0.7)
    
    ## this barplot is for RSEM of gene expression
    barplot(log2(junctionGeneRSEM[j,samples]+1),las=2, cex.names=0.3,
            col=ifelse(pheno[samples,'classes']=='Tumor',tumcol,normcol))
    title(sprintf('%s \nRSEM gene expression',geneAnnot[j,]$SYMBOL), cex.main=0.7)
    
    count=count+1
    print(count)
    ## this fills the last section of the plot with the gene data
    info<-GetGeneInfo(HPVOPEntrezID[which(HPVOPgenenames==geneAnnot[j,]$SYMBOL)])
    if (is.null(info))
    { textplot('No gene data', cex=1)
      title(geneAnnot[j,]$SYMBOL, cex=1)
      title(paste('Junction in', geneAnnot[j,]$SYMBOL,'\n', j), 3, outer=T, line=-1, cex=0.9)
      next
    }
    if (info$genesummary>1)
    {
      textplot(strwrap(info$genesummary, width=50), valign='top', halign='center', cex=0.7, fixed.width = F)
      title(strwrap(sprintf(info$genename), width=35), cex.main=0.8)
    }else{
      textplot(sprintf(geneAnnot[j,]$SYMBOL), cex=1)
      title(strwrap(sprintf(info$genename), width=35), cex.main=0.8)
    }
    title(paste('Junction in', geneAnnot[j,]$SYMBOL,'\n', j), 3, outer=T, line=-1, cex=0.9)
  }
  
  
  
  if (makepdf==T) dev.off()

